#include "reality_gap_loop_functions.h"
#include <argos3/core/simulator/simulator.h>
#include <argos3/core/utility/configuration/argos_configuration.h>
#include <argos3/plugins/robots/e-puck/simulator/epuck_entity.h>
#include "epuck_reality_gap.h"


/****************************************/
/****************************************/

CRealityGapLoopFunctions::CRealityGapLoopFunctions() 
{
}

/****************************************/
/****************************************/

void CRealityGapLoopFunctions::Init(TConfigurationNode& t_node) {
    try {
        TConfigurationNode& tRealityGap = GetNode(t_node, "tcpconnection");
        GetNodeAttributeOrDefault(tRealityGap, "connect", m_bConnectToServer, false);
        GetNodeAttribute(tRealityGap, "host", m_sTCPHost);
    }
    catch(CARGoSException& ex) {
      THROW_ARGOSEXCEPTION_NESTED("Error parsing loop functions!", ex);
   }

    if (m_bConnectToServer) {
       m_pcTCPConnection = new TCPConnection(m_sTCPHost, 4321);
        m_pcTCPConnection->Start();
    }

    m_unNbCs = 0;
    m_bExperimentStarted = false;
}

/****************************************/
/****************************************/

void CRealityGapLoopFunctions::Reset() {
}

/****************************************/
/****************************************/

void CRealityGapLoopFunctions::Destroy() {
}

/****************************************/
/****************************************/

CColor CRealityGapLoopFunctions::GetFloorColor(const CVector2& c_position_on_plane) {
   return CColor::WHITE;
}



/****************************************/
/****************************************/

void CRealityGapLoopFunctions::PreStep() {
    if (m_bConnectToServer) {
        ReadIncomingTCPMessages();
        if (m_bExperimentStarted) {
            m_unNbCs++;
        }
    }
}


/****************************************/
/****************************************/
void CRealityGapLoopFunctions::ReadIncomingTCPMessages() {
    std::string sMsg = m_pcTCPConnection->GetLatestMessage();
    if (sMsg == "a") {
        CSpace::TMapPerType& cEpucks = GetSpace().GetEntitiesByType("epuck");

            for(CSpace::TMapPerType::iterator it = cEpucks.begin(); it != cEpucks.end(); ++it) {
                CEPuckEntity& cEpuck = *any_cast<CEPuckEntity*>(it->second);
                CEpuckRealityGap& cController = dynamic_cast<CEpuckRealityGap&>(cEpuck.GetControllableEntity().GetController());

                cController.m_bWaitForExperimentToStart = false;
                m_bExperimentStarted = true;
                
        }

    }
}
/****************************************/
/****************************************/

void CRealityGapLoopFunctions::PostStep() {

  

}


/****************************************/
/****************************************/
bool CRealityGapLoopFunctions::IsExperimentFinished() {
    return (m_unNbCs >= 600);
}
/****************************************/
/****************************************/

REGISTER_LOOP_FUNCTIONS(CRealityGapLoopFunctions, "reality_gap1_loop_functions")
